@@ -4,7 +4,7 @@ import functools
 import io
 import threading
 from abc import ABC, abstractmethod
-from collections import deque
+from collections import defaultdict, deque
 from concurrent.futures import Future, ThreadPoolExecutor
 from pathlib import Path
 from types import TracebackType
@@ -176,7 +176,7 @@ class PyAVReader(BasePyAVReader):
             if len(self.streams) < 2:
                 self._alpha_merger = NotAlphaMerger()
             elif len(self.streams) == 2:
-                self._alpha_merger = AlphaMerger(*self.streams)
+                self._alpha_merger = AlphaMerger()
             else:
                 raise NotImplementedError
         return self._alpha_merger
@@ -257,6 +257,7 @@ class PyAVWriter(PyAVInterface):
         audio_sample_rate=48000,
         audio_format="s16",
         audio_layout="stereo",
+        audio_bit_rate=192000,
         format=None,
         options={},
     ):
@@ -291,6 +292,7 @@ class PyAVWriter(PyAVInterface):
         self._audio_sample_rate = audio_sample_rate
         self._audio_format = audio_format
         self._audio_layout = audio_layout
+        self._audio_bit_rate = audio_bit_rate
         self._format = format
         self._options = options
 
@@ -352,6 +354,7 @@ class PyAVWriter(PyAVInterface):
             )
             audio_stream.format = self._audio_format
             audio_stream.layout = self._audio_layout
+            audio_stream.bit_rate = self._audio_bit_rate
 
         self._streams = streams
         self._audio_stream = audio_stream
@@ -465,34 +468,52 @@ class PyAVWriter(PyAVInterface):
 
 
 class Filter(ABC):
-    graph: av.filter.Graph
+    graph: av.filter.Graph | None
 
     def __init__(self):
-        self.graph = av.filter.Graph()
+        self.graph = None
+        self._kwargs = {}
+
+    def rebuild_graph(self, **kwargs):
+        if self._kwargs != kwargs:
+            self.close()
+            self.graph = av.filter.Graph()
+            self._rebuild_graph(**kwargs)
+            self.graph.configure()
+
+    @abstractmethod
+    def _rebuild_graph(**kwargs):
+        "rebuild graph impl"
 
     def close(self):
-        self.graph.push(None)
+        if self.graph is not None:
+            self.graph.push(None)
 
 
 class _Formatter(Filter):
-    def __init__(self, width: int, height: int, from_pix_fmt: str, to_pix_format: str):
+    def __init__(self, to_pix_fmt: str):
         super().__init__()
+
+        self._to_pix_fmt = to_pix_fmt
+
+    def _rebuild_graph(self, width: int, height: int, pix_fmt: str):
         src = self.graph.add_buffer(
             width=width,
             height=height,
-            format=from_pix_fmt,
+            format=pix_fmt,
             time_base=fractions.Fraction(1, 1000),
         )
 
-        reformat = self.graph.add("format", to_pix_format)
+        reformat = self.graph.add("format", self._to_pix_fmt)
         src.link_to(reformat)
 
         sink = self.graph.add("buffersink")
         reformat.link_to(sink)
 
-        self.graph.configure()
-
     def __call__(self, frame: av.VideoFrame) -> av.VideoFrame:
+        self.rebuild_graph(
+            width=frame.width, height=frame.height, pix_fmt=frame.format.name
+        )
         self.graph.push(frame)
         ret = self.graph.pull()
         ret.pts = None
@@ -504,9 +525,7 @@ def to_rgba(reader: Iterator[av.VideoFrame]):
 
     for frame in reader:
         if frame.format.name not in formatter:
-            formatter[frame.format.name] = _Formatter(
-                frame.width, frame.height, frame.format.name, "rgba"
-            )
+            formatter[frame.format.name] = _Formatter("rgba")
 
         yield formatter[frame.format.name](frame)
 
@@ -520,8 +539,14 @@ def to_array(iterator: list[av.VideoFrame]):
 
 
 class _AlphaExtractor(Filter):
-    def __init__(self, width: int, height: int, pix_fmt: str):
+    def __init__(self):
         super().__init__()
+
+        self._width = None
+        self._height = None
+        self._pix_fmt = None
+
+    def _rebuild_graph(self, width: int, height: int, pix_fmt: str):
         src = self.graph.add_buffer(
             width=width,
             height=height,
@@ -535,31 +560,26 @@ class _AlphaExtractor(Filter):
         alpha = self.graph.add("buffersink")
         alphaextract.link_to(alpha)
 
-        self.graph.configure()
-
     def __call__(self, frame: av.VideoFrame):
+        self.rebuild_graph(
+            width=frame.width, height=frame.height, pix_fmt=frame.format.name
+        )
         self.graph.push(frame)
         return self.graph.pull()
 
 
 class AlphaExtractor:
-    def __init__(self, width: int, height: int):
-        assert height % 2 == 0
-
-        self.rgba = _AlphaExtractor(width, height, "rgba")
-        self.yuva420p = _AlphaExtractor(width, height, "yuva420p")
+    def __init__(self):
+        self.alpha_extractors = defaultdict(_AlphaExtractor)
 
     def __call__(self, frame: av.VideoFrame):
-        if frame.format.name == "rgba":
-            return self.rgba(frame)
-        elif frame.format.name == "yuva420p":
-            return self.yuva420p(frame)
-        else:
-            raise NotImplementedError
+        assert frame.height % 2 == 0
+
+        return self.alpha_extractors[frame.format.name](frame)
 
     def close(self):
-        self.rgba.close()
-        self.yuva420p.close()
+        for it in self.alpha_extractors.values():
+            it.close()
 
 
 class BaseAlphaMerger:
@@ -576,12 +596,53 @@ class BaseAlphaMerger:
         """pull merged image"""
 
 
-class AlphaMerger(Filter, BaseAlphaMerger):
-    def __init__(self, template1: av.VideoStream, template2: av.VideoStream):
-        super().__init__()
+class AlphaMerger(BaseAlphaMerger):
+    def __init__(self):
+        self.alpha_mergers = defaultdict(_AlphaMerger)
+
+    def push_image(self, frame: av.VideoFrame):
+        self.alpha_mergers[(frame.width, frame.height, frame.format.name)].push_image(
+            frame
+        )
+
+    def push_alpha(self, frame: av.VideoFrame):
+        self.alpha_mergers[(frame.width, frame.height, frame.format.name)].push_alpha(
+            frame
+        )
+
+    def pull(self) -> av.VideoFrame:
+        for key in list(self.alpha_mergers):
+            ret = self.alpha_mergers[key].pull()
+            if ret is not None:
+                return ret
+
+            del self.alpha_mergers[key]
+
+    def close(self):
+        for it in self.alpha_mergers.values():
+            it.close()
+
+
+class _AlphaMerger(Filter, BaseAlphaMerger):
+    def _rebuild_graph(
+        self,
+        width: int,
+        height: int,
+        pix_fmt: str,
+    ):
+        self.image = self.graph.add_buffer(
+            width=width,
+            height=height,
+            format=pix_fmt,
+            time_base=fractions.Fraction(1, 1000),
+        )
+        self.alpha = self.graph.add_buffer(
+            width=width,
+            height=height,
+            format=pix_fmt,
+            time_base=fractions.Fraction(1, 1000),
+        )
 
-        self.image = self.graph.add_buffer(template=template1)
-        self.alpha = self.graph.add_buffer(template=template2)
         format = self.graph.add("format", "gray")
         self.alpha.link_to(format)
 
@@ -592,12 +653,16 @@ class AlphaMerger(Filter, BaseAlphaMerger):
         self.result = self.graph.add("buffersink")
         alphamerge.link_to(self.result)
 
-        self.graph.configure()
-
     def push_image(self, frame: av.VideoFrame):
+        self.rebuild_graph(
+            width=frame.width, height=frame.height, pix_fmt=frame.format.name
+        )
         self.image.push(frame)
 
     def push_alpha(self, frame: av.VideoFrame):
+        self.rebuild_graph(
+            width=frame.width, height=frame.height, pix_fmt=frame.format.name
+        )
         self.alpha.push(frame)
 
     def pull(self) -> av.VideoFrame:
@@ -677,8 +742,6 @@ class BaseOverlayer(Filter):
         background_image: np.ndarray,
         x: int,
         y: int,
-        w: int,
-        h: int,
         pix_fmt: str = "yuva420p",
         mode: str = "straight",
         pre_overlay_filter: Callable[
@@ -698,10 +761,20 @@ class BaseOverlayer(Filter):
             background_image,
             format="rgb24" if background_image.shape[-1] == 3 else "rgba",
         )
+        self.x = x
+        self.y = y
+        self.pix_fmt = pix_fmt
+        self.mode = mode
 
+    def _rebuild_graph(
+        self,
+        width: int,
+        height: int,
+        pix_fmt: str,
+    ):
         self.src = self.graph.add_buffer(
-            width=w,
-            height=h,
+            width=width,
+            height=height,
             format=pix_fmt,
             time_base=fractions.Fraction(1, 1000),
         )
@@ -714,11 +787,13 @@ class BaseOverlayer(Filter):
 
         pre_filtered = self.pre_overlay_filter(self.graph, self.src)
 
-        overlay = self.graph.add("overlay", f"x={x}:y={y}:alpha={mode}:format=auto")
+        overlay = self.graph.add(
+            "overlay", f"x={self.x}:y={self.y}:alpha={self.mode}:format=auto"
+        )
         self.dst.link_to(overlay, input_idx=0)
         pre_filtered.link_to(overlay, input_idx=1)
 
-        format = self.graph.add("format", pix_fmt)
+        format = self.graph.add("format", self.pix_fmt)
         overlay.link_to(format)
 
         post_filtered = self.post_overlay_filter(self.graph, format)
@@ -726,11 +801,12 @@ class BaseOverlayer(Filter):
         sink = self.graph.add("buffersink")
         post_filtered.link_to(sink)
 
-        self.graph.configure()
-
-    def paste(self, src_image):
-        src_image.pts = None
-        self.src.push(src_image)
+    def paste(self, frame: av.VideoFrame):
+        self.rebuild_graph(
+            width=frame.width, height=frame.height, pix_fmt=frame.format.name
+        )
+        frame.pts = None
+        self.src.push(frame)
         self.dst.push(self.background_image)
         ret = self.graph.pull()
         ret.pts = None
@@ -759,10 +835,9 @@ class Overlayer(BaseOverlayer):
         left=0.0,
         top=0.0,
         height=1.0,
-        pix_fmt: str = "yuva420p",
+        pix_fmt="yuva420p",
         mode: str = "straight",
     ):
-        origin_src_size = src_size
         dst_size, background_image = get_dst_size(dst_size, background_image)
         src_pos, src_size = get_src_size(left, top, height, dst_size, src_size)
 
@@ -772,8 +847,6 @@ class Overlayer(BaseOverlayer):
             background_image,
             x=src_pos[0],
             y=src_pos[1],
-            w=origin_src_size[0],
-            h=origin_src_size[1],
             pix_fmt=pix_fmt,
             mode=mode,
             pre_overlay_filter=self._pre_overlay_filter,
